# Modul za crtanje funkcija: crt_funk
'''Modul je smješten u mapi C:\...\moj_py.
   Prije njegova pozivanja treba tu mapu aktivirati s
       import sys
       sys.path.append('C:\...\moj_py')'''

from turtle import *

def okvir(x_dim, y_dim):
    '''Funkcija crta pravokutnik dimenzija:
          x_dim (jedinica)
          y_dim (jedinica)
        središnje smješten u grafičkom prozoru'''
    ht(); pu(); speed(0)
    x_p = -x_dim // 2; y_p = -y_dim // 2
    goto(x_p, y_p)
    pd(); pencolor('black')
    fd(x_dim); lt(90); fd(y_dim); lt(90); fd(x_dim); lt(90); fd(y_dim); lt(90)
    return

def podjela_osi(os_dim, oznake):
    '''Funkcija određuje listu ordinata podjeljaka.
    Razmaci su podešeni kako se ne bi uočila pogreška zbog zaokruživanja'''
    pd = len(oznake) - 1
    d = round(os_dim / pd)
    razlika = os_dim -  d * pd
    os = [-os_dim // 2]
    for i in range(1, pd + 1):
        os.append(os[i-1] + d)
        if razlika != 0 and (i % (pd // abs(razlika)) == 0):
            if razlika > 0:
                os[i] += 1
            else:
                os[i] -= 1
    return os

def crtanje_x_osi(x_dim, y_dim, oz_x, opis_x, mreža):
    ''' Funkcija crta podjele i ispisuje oznake uz os x.
       x_dim   duljina x osi izražena u pikselima (int)
       y_dim   duljina y osi izražena u pikselima (int)
       oz_x    lista oznaka na podjeljcima x osi, elementi liste su brojevi,
               na mjestima gdje dolazi podjeljak bez oznaka stavlja se umjesto
               broja prazni string ''
       opis_x  opis osi x (string)
       mreža   1 crta se koordinatna mreža
               0 mreža se ne crta'''       
    x_os = podjela_osi(x_dim, oz_x)
    # crtanje podjeljaka i oznaka na osi x
    seth(90)
    y_p = -y_dim // 2
    for i in range(1, len(x_os) - 1):
        pu(); goto(x_os[i], y_p)
        pd(); fd(5)
        if mreža == 1:
            pencolor(0.8, 0.8, 0.8) 
            fd(y_dim - 5)
            pencolor('black')
    for i in range(len(x_os)):
        pu(); goto(x_os[i], y_p - 20)
        write(oz_x[i], align = 'center') 
    # opis osi x
    pu(); goto(x_dim // 2 - len(opis_x), y_p - 40)
    write(opis_x, align = 'right', font = ('Arial', 10, 'normal'))
    return

def crtanje_y_osi(x_dim, y_dim, oz_y, opis_y, mreža):
    '''Funkcija crta podjele i ispisuje oznake uz os y
       x_dim   duljina x osi izražena u pikselima (int)
       y_dim   duljina y osi izražena u pikselima (int)
       oz_y    lista oznaka na podjeljcima y osi, elementi liste su brojevi,
               na mjestima gdje dolazi podjeljak bez oznaka stavlja se umjesto
               broja prazni string ''
       opis_y  opis osi y (string)
       mreža   1 crta se koordinatna mreža
               0 mreža se ne crta'''
    y_os = podjela_osi(y_dim, oz_y)
    # crtanje podjeljaka na osi y
    seth(0) 
    x_p = -x_dim // 2; y_p = -y_dim // 2
    for i in range(1, len(y_os) - 1):
        pu(); goto(x_p, y_os[i])
        pd(); fd(5)
        if mreža == 1:
            pencolor(0.8, 0.8, 0.8)
            fd(x_dim - 5)
            pencolor('black')
    for i in range(len(y_os)):
        pu(); goto(x_p - 8 - len(str(oz_y[i])), y_os[i] - 7)
        write(oz_y[i], align = 'right')
    # opis osi y
    pu(); goto(x_p - 8 - len(opis_y), y_dim // 2 + 10)
    write(opis_y, align = 'center', font = ('Arial', 10, 'normal'))
    return

def koord_sustav(naziv, oz_x, oz_y, op_x, op_y, x_dim, y_dim, mreža):
    '''Funkcija crta koordinatni sustav
       naziv   string
       x_dim   duljina x osi izražena u pikselima (int)
       y_dim   duljina y osi izražena u pikselima (int)
       oz_x    lista oznaka na podjeljcima x osi, elementi liste su brojevi,
               na mjestima gdje dolazi podjeljak bez oznaka stavlja se umjesto
               broja prazni string ''. Prvi i zadnji element liste moraju biti
               brojevi.
       oz_y    lista oznaka na podjeljcima y osi, elementi liste su brojevi,
               na mjestima gdje dolazi podjeljak bez oznaka stavlja se umjesto
               broja prazni string ''. Prvi i zadnji element liste moraju biti
               brojevi.
       opis_x  opis osi x (string)
       opis_y  opis osi y (string)
       mreža   1 crta se koordinatna mreža
               0 mreža se ne crta'''

    title('Crtanje funkcije ' + naziv)
    okvir(x_dim, y_dim)
    crtanje_x_osi(x_dim, y_dim, oz_x, op_x, mreža)
    crtanje_y_osi(x_dim, y_dim, oz_y, op_y, mreža)
    pu(); goto(0, y_dim // 2 + 20)
    write(naziv, align = 'center', font = ('Arial', 11, 'normal'))
    delta_x = (oz_x[-1] - oz_x[0]) / x_dim
    x_d = [oz_x[0] + i * delta_x for i in range(x_dim + 1)]
    return x_d

def crtanje_funkcije(y_k, oznake_x, oznake_y, x_dim, y_dim):
    '''Funkcija iz liste kontinuiranih vrijednosti funkcije y_k
    izračunava pripadne kvantizirane vrijednosti y_d i zatim povlači liniju
    funkcije kroz točke (x_d[i], y_d[i].
        y_k     lista kontinuiranih vrijednosti za koju je y_k[i]=funk(x_d[i])
        x_dim   duljina x osi izražena u pikselima (int)
        y_dim   duljina y osi izražena u pikselima (int) 
        oz_y    lista oznaka na podjeljcima y osi, elementi liste su brojevi,
                na mjestima gdje dolazi podjeljak bez oznaka stavlja se umjesto
                broja prazni string ''        '''
    delta_y = (oznake_y[-1] - oznake_y[0]) / y_dim
    sredina_y = (oznake_y[0] + oznake_y[-1]) / 2
    y_d = [0.0 for i in range(x_dim + 1)]
    for i in range(x_dim + 1):
        y_d[i] = round((y_k[i] - sredina_y) / delta_y)
    for i in range(x_dim + 1):
            if y_d[i] > y_dim // 2: 
                 y_d[i] = y_dim // 2
            if y_d[i] < - y_dim // 2: 
                 y_d[i] = - y_dim // 2
    pu(); goto(-x_dim // 2, y_d[0]); pd()
    for i in range(x_dim + 1):
        goto(-x_dim // 2 + i, y_d[i])
    return




